@@ -421,7 +421,7 @@ build_request (cherokee_handler_proxy_t *hdl,
 	}
 	else {
 		cherokee_buffer_add_buffer (buf, &hdl->src_ref->host);
-		if (hdl->src_ref->port != 80) {
+		if (! http_port_is_standard (hdl->src_ref->port, conn->socket.is_tls)) {
 			cherokee_buffer_add_char    (buf, ':');
 			cherokee_buffer_add_ulong10 (buf, hdl->src_ref->port);
 		}
@@ -568,8 +568,7 @@ build_request (cherokee_handler_proxy_t *hdl,
 		cherokee_buffer_add_str    (buf, "X-Forwarded-Host: ");
 		cherokee_buffer_add_buffer (buf, &conn->host);
 
-		if (((! conn->socket.is_tls) && (conn->bind->port != 80)) ||
-		    ((  conn->socket.is_tls) && (conn->bind->port != 443)))
+		if (! http_port_is_standard (conn->bind->port, conn->socket.is_tls))
 		{
 			cherokee_buffer_add_char    (buf, ':');
 			cherokee_buffer_add_ulong10 (buf, conn->bind->port);
