@@ -75,8 +75,6 @@ CHEROKEE_BEGIN_DECLS
 # define cherokee_error            GetLastError()
 # define cherokee_mkdir(path,perm) mkdir(path)
 #else
-# define cherokee_stat(path,buf)   stat(path,buf)
-# define cherokee_lstat(path,buf)  lstat(path,buf)
 # define cherokee_error            errno
 # define cherokee_mkdir(path,perm) mkdir(path,perm)
 #endif
@@ -147,6 +145,9 @@ long      *cherokee_get_timezone_ref (void);
 /* Thread safe functions
  */
 int   cherokee_readdir       (DIR *dirstream, struct dirent *entry, struct dirent **result);
+int   cherokee_stat          (const char *restrict path, struct stat *buf);
+int   cherokee_lstat         (const char *restrict path, struct stat *buf);
+int   cherokee_fstat         (int filedes, struct stat *buf);
 ret_t cherokee_gethostbyname (const char *hostname, void *addr);
 ret_t cherokee_gethostname   (cherokee_buffer_t *buf);
 ret_t cherokee_syslog        (int priority, cherokee_buffer_t *buf);
@@ -156,6 +157,7 @@ ret_t cherokee_mkstemp       (cherokee_buffer_t *buffer, int *fd);
 ret_t cherokee_mkdir_p       (cherokee_buffer_t *path, int mode);
 ret_t cherokee_ntop          (int family, struct sockaddr *addr, char *dst, size_t cnt);
 ret_t cherokee_mkdir_p_perm  (cherokee_buffer_t *dir_path, int create_mode, int ensure_perm);
+ret_t cherokee_wait_pid      (int pid, int *retcode);
 
 ret_t cherokee_io_stat       (cherokee_iocache_t        *iocache,
 			      cherokee_buffer_t         *path,
